@@ -1,17 +1,50 @@
+using System;
 using System.IO;
+using FSpot.Xmp;
 
 namespace FSpot {
 	public interface IThumbnailContainer {
 		Gdk.Pixbuf GetEmbeddedThumbnail ();
 	}
 
-	public class JpegFile : ImageFile, IThumbnailContainer {
+	public class JpegFile : ImageFile, IThumbnailContainer, SemWeb.StatementSource {
 		private Exif.ExifData exif_data;
+		private XmpFile xmp;
+		private JpegHeader header;
 		
-		public JpegFile (string path) : base (path) {}
+		public JpegFile (string path) : base (path) 
+		{
+#if false // TEST_METADATA
+			MetadataStore store = new MetadataStore ();
+			Select (store);
+			store.Dump ();
+#endif
+		}
 
-		public string Description {
+		public JpegHeader Header {
 			get {
+				if (header == null) {
+					using (System.IO.FileStream stream = System.IO.File.OpenRead (path)) {
+						header = new JpegHeader (stream, true);
+					}
+				}
+				return header;
+			}
+		}
+
+		public void Select (SemWeb.StatementSink sink)
+		{
+			Header.Select (sink);
+		}
+
+		public override Cms.Profile GetProfile ()
+		{
+			return Header.GetProfile ();
+		}
+
+		public override string Description {
+			get {
+#if true
 				// FIXME this should probably read the raw data because libexif sucks.
 				Exif.ExifContent exif_content = this.ExifData.GetContents (Exif.Ifd.Exif);
 				Exif.ExifEntry entry = exif_content.Lookup (Exif.Tag.UserComment);
@@ -20,20 +53,39 @@ namespace FSpot {
 					return null;
 				
 				return entry.Value;
-			}
-			set {
-				string description = value;
+#else
 
-				Exif.ExifContent exif_content = this.ExifData.GetContents (Exif.Ifd.Exif);			
-				int len = System.Text.Encoding.BigEndianUnicode.GetByteCount (description);
-				string heading = "ASCII\0\0\0";
-				byte [] data = new byte [len + heading.Length];
-				System.Text.Encoding.ASCII.GetBytes (heading, 0, heading.Length, data, 0);
-				System.Text.Encoding.ASCII.GetBytes (description, 0, description.Length, data, heading.Length);
-				exif_content.GetEntry (Exif.Tag.UserComment).SetData (data);
+#endif
 			}
 		}
 
+		public void SetDescription (string value)
+		{
+			string description = value;
+#if USE_UNICODE_COMMENTS		
+			Exif.ExifContent exif_content = this.ExifData.GetContents (Exif.Ifd.Exif);			
+			int len = System.Text.Encoding.BigEndianUnicode.GetByteCount (description);
+			string heading = "UNICODE\0";
+			byte [] data = new byte [len + heading.Length];
+			System.Text.Encoding.ASCII.GetBytes (heading, 0, heading.Length, data, 0);
+			System.Text.Encoding.BigEndianUnicode.GetBytes (description, 0, description.Length, data, heading.Length);
+			exif_content.GetEntry (Exif.Tag.UserComment).SetData (data);
+#else
+			Exif.ExifContent exif_content = this.ExifData.GetContents (Exif.Ifd.Exif);			
+			int len = System.Text.Encoding.ASCII.GetByteCount (description);
+			string heading = "ASCII\0\0\0";
+			byte [] data = new byte [len + heading.Length];
+			System.Text.Encoding.ASCII.GetBytes (heading, 0, heading.Length, data, 0);
+			System.Text.Encoding.ASCII.GetBytes (description, 0, description.Length, data, heading.Length);
+			exif_content.GetEntry (Exif.Tag.UserComment).SetData (data);
+#endif
+		}
+		
+		public void SetXmp (XmpFile xmp)
+		{
+			this.xmp = xmp;
+		}
+
 		private void UpdateMeta ()
 		{
 			Exif.ExifContent image_content = this.ExifData.GetContents (Exif.Ifd.Zero);
@@ -47,8 +99,12 @@ namespace FSpot {
 		{
 			JpegHeader header = new JpegHeader (input);
 			UpdateMeta ();
+			
+			header.SetExif (this.ExifData);
+
+			if (xmp != null)
+				header.SetXmp (xmp);
 
-			header.Exif = this.ExifData;
 			header.Save (output);
 		}
 		
@@ -56,9 +112,6 @@ namespace FSpot {
 		{
 			UpdateMeta ();
 
-			//this.ExifData.Dump ();
-
-#if true //USE_UNSTABLE_JPEG_HEADER_CODE
 			string  temp_path = path;
 			using (System.IO.FileStream stream = System.IO.File.OpenRead (path)) {
 				using (System.IO.Stream output = FSpot.Unix.MakeSafeTemp (ref temp_path)) {
@@ -69,10 +122,6 @@ namespace FSpot {
 				System.IO.File.Delete (temp_path);
 				throw new System.Exception (System.String.Format ("Unable to rename {0} to {1}", temp_path, path));
 			}
-#else 
-			JpegUtils.SaveExif (path, this.ExifData);
-#endif
-
 		}
 		
 		public override void Save (Gdk.Pixbuf pixbuf, System.IO.Stream stream)
@@ -168,19 +217,51 @@ namespace FSpot {
 			e.SetData ((ushort)orientation);
 		}
 		
-		public override System.DateTime Date () {
-			System.DateTime time;
-			try {
-				using (Exif.ExifData ed = new Exif.ExifData (path)) {
-					Exif.ExifContent content = ed.GetContents (Exif.Ifd.Exif);
-					Exif.ExifEntry entry = content.GetEntry (Exif.Tag.DateTimeOriginal);
-					time = Exif.ExifUtil.DateTimeFromString (entry.Value); 
-					time = time.ToUniversalTime ();
+		public void SetDateTimeOriginal (DateTime time)
+		{
+			Exif.ExifEntry e = ExifData.LookupFirst (Exif.Tag.DateTimeOriginal);
+			if (e != null)
+				e.SetData (time);
+		}
+
+		public override System.DateTime Date {
+			get {
+				System.DateTime time;
+				try {
+#if true
+					using (Exif.ExifData ed = new Exif.ExifData (path)) {
+						string time_str = "";				
+						time_str = ed.LookupFirstValue (Exif.Tag.DateTimeOriginal);
+
+						if (time_str == null || time_str == "") 
+							time_str = ed.LookupFirstValue (Exif.Tag.DateTime);
+
+						time = Exif.ExifUtil.DateTimeFromString (time_str).ToUniversalTime (); 
+#else
+						TiffHeader tiff = Header.GetExifHeader ();
+						SubdirectoryEntry sub = (SubdirectoryEntry) tiff.Directory.Lookup (TagId.ExifIfdPointer);
+						DirectoryEntry e;
+						
+						if (sub != null) {
+							e = sub.Directory [0].Lookup (TagId.DateTimeOriginal);
+							
+							if (e != null)
+								return DirectoryEntry.DateTimeFromString (e.StringValue);
+						}
+						
+						e = tiff.Directory.Lookup (TagId.DateTime);
+
+						if (e != null)
+							return DirectoryEntry.DateTimeFromString (e.StringValue);
+						
+						return base.Date;
+#endif
+					}
+				} catch (System.Exception e) {
+					time = base.Date;
 				}
-			} catch (System.Exception e) {
-				time = base.Date ();
+				return time;
 			}
-			return time;
 		}
 		
 	}
