@@ -1,5 +1,8 @@
 using ICSharpCode.SharpZipLib.Zip.Compression;
 using SemWeb;
+using Cms;
+using System.IO;
+using FSpot.Xmp;
 
 namespace FSpot.Png {
 	public class PngFile : ImageFile, SemWeb.StatementSource {
@@ -51,6 +54,7 @@ namespace FSpot.Png {
 						MetadataStore.AddLiteral (sink, "exif:UserComment", text.Text);
 						break;
 					case "Software":
+						MetadataStore.AddLiteral (sink, "xmp:CreatorTool", text.Text);
 						break;
 					case "Title":
 						MetadataStore.AddLiteral (sink, "dc:title", "rdf:Alt", new Literal (text.Text, "x-default", null));
@@ -134,7 +138,27 @@ namespace FSpot.Png {
 				}
 			}
 
-			public ZtxtChunk (string name, byte [] data) : base (name, data) {}
+			public ZtxtChunk (string keyword, string text) : base ()
+			{
+				Name = "zTXt";
+				Compression = 0;
+				this.keyword = keyword;
+			}
+
+			public ZtxtChunk (string name, byte [] data) : base (name, data)
+			{
+			}			
+
+			protected ZtxtChunk ()
+			{
+			}
+
+			public override void SetText (string text)
+			{
+				/* FIXME this is broken */
+				text_data = encoding.GetBytes (text);
+				data = Chunk.Deflate (text_data, 0, text_data.Length);
+			}
 			
 			public override void Load (byte [] data) 
 			{
@@ -173,12 +197,26 @@ namespace FSpot.Png {
 			//public static string Name = "tEXt";
 
 			protected string keyword;
-			protected string text;
 			protected byte [] text_data;
 			protected System.Text.Encoding encoding = Latin1;
 
 			public static System.Text.Encoding Latin1 = System.Text.Encoding.GetEncoding (28591);
-			public TextChunk (string name, byte [] data) : base (name, data) {}
+
+			public TextChunk (string name, byte [] data) : base (name, data) 
+			{
+			}
+
+			protected TextChunk ()
+			{
+			}
+
+
+			public TextChunk (string keyword, string text)
+			{
+				this.Name = "tEXt";
+				this.keyword = keyword;
+				SetText (text);
+			}
 
 			public override void Load (byte [] data)
 			{
@@ -197,13 +235,23 @@ namespace FSpot.Png {
 				}
 			}
 
-			public byte [] TextData 
-			{
+			public byte [] TextData {
 				get {
 					return text_data;
 				}
 			}
 			
+			public virtual void SetText (string text)
+			{
+				text_data = encoding.GetBytes (text);
+
+				byte [] keyword_data = Latin1.GetBytes (keyword);
+				data = new byte [keyword_data.Length + 1 + text_data.Length];
+				System.Array.Copy (keyword_data, 0, data, 0, keyword_data.Length);
+				data [keyword_data.Length] = 0;
+				System.Array.Copy (text_data, 0, data, keyword_data.Length + 1, text_data.Length);
+			}
+
 			public string Text {
 				get {
 					return encoding.GetString (text_data, 0, text_data.Length);
@@ -269,10 +317,63 @@ namespace FSpot.Png {
 				}
 			}
 
+			public override void SetText (string text)
+			{
+				byte [] raw = System.Text.Encoding.UTF8.GetBytes (text);
+				SetText (raw);
+			}
+
+			public void SetText (byte [] raw)
+			{
+				MemoryStream stream = new MemoryStream ();
+				byte [] tmp;
+
+				text_data = raw;
+
+				tmp = Latin1.GetBytes (keyword);
+				stream.Write (tmp, 0, tmp.Length);
+				stream.WriteByte (0);
+
+				stream.WriteByte ((byte)(compressed ? 1 : 0));
+				stream.WriteByte (Compression);
+
+				if (Language != null && Language != "") {
+					tmp = Latin1.GetBytes (Language);
+					stream.Write (tmp, 0, tmp.Length);
+				}
+				stream.WriteByte (0);
+
+				if (LocalizedKeyword != null && LocalizedKeyword != "") {
+					tmp = System.Text.Encoding.UTF8.GetBytes (LocalizedKeyword);
+					stream.Write (tmp, 0, tmp.Length);
+				}
+				stream.WriteByte (0);
+				
+				if (compressed) {
+					tmp = Deflate (text_data, 0, text_data.Length);
+					stream.Write (tmp, 0, tmp.Length);
+				} else {
+					stream.Write (text_data, 0, text_data.Length);
+				}
+				this.data = stream.ToArray ();
+			}
+
 			public ItxtChunk (string name, byte [] data) : base (name, data) 
 			{
+				this.Name = name;
 				encoding = System.Text.Encoding.UTF8;
 			}
+
+			public ItxtChunk (string keyword, string language, bool compressed) : base ()
+			{
+				encoding = System.Text.Encoding.UTF8;
+				this.Name = "iTXt";
+				this.keyword = keyword;
+				this.Language = language;
+				this.LocalizedKeyword = "";
+				this.compressed = compressed;
+				this.Compression = 0;
+			}
 		}
 
 		public class TimeChunk : Chunk {
@@ -299,8 +400,35 @@ namespace FSpot.Png {
 			}
 			
 			public TimeChunk (string name, byte [] data) : base (name, data) {}
+			
+			public TimeChunk ()
+			{
+				this.Name = "tIME";
+				this.Time = System.DateTime.Now;
+			}
 		}
 
+		public class StandardRgbChunk : Chunk {
+			public StandardRgbChunk (string name, byte [] data) : base (name, data) {}
+			
+			public Cms.Intent RenderingIntent {
+				get {
+					return (Cms.Intent) data [0];
+				}
+			}
+		}
+
+		public class GammaChunk : Chunk {
+			public GammaChunk (string name, byte [] data) : base (name, data) {}
+			private const int divisor = 100000;
+
+			public double Gamma {
+				get {
+					return FSpot.BitConverter.ToUInt32 (data, 0, false) / (double) divisor;
+				}
+			}
+		}
+		
 		public class ColorChunk : Chunk {
 			// FIXME this should be represented like a tiff rational
 			public const uint Denominator = 100000;
@@ -448,6 +576,69 @@ namespace FSpot.Png {
 			}
 		}
 
+		public class Crc {
+			static uint [] lookup;
+			uint value = 0xffffffff;
+			uint length;
+			System.IO.Stream stream;
+
+			public uint Value {
+				get { return (value ^ 0xffffffff); }
+			}
+
+			public uint Length {
+				get { return length; }
+			}
+
+			static Crc () {
+				lookup = new uint [265];
+				uint c, n;
+				int k;
+				
+				for (n = 0; n < 256; n++) {
+					c = n;
+					for (k = 0; k < 8; k++) {
+						if ((c & 1) != 0)
+							c = 0xedb88320 ^ (c >> 1);
+						else
+							c = c >> 1;
+					}
+					lookup [n] = c;
+				}
+			}
+
+			public Crc ()
+			{
+			}
+
+			public Crc (System.IO.Stream stream)
+			{
+				this.stream = stream;
+			}
+			
+			public void Write (byte [] buffer)
+			{
+				Write (buffer, 0, buffer.Length);
+			}
+
+			public void Write (byte [] buffer, int offset, int len)
+			{
+				for (int i = offset; i < len; i++) 
+					value = lookup [(value ^ buffer[i]) & 0xff] ^ (value >> 8); 
+
+				length += (uint)len;
+
+				if (stream != null)
+					stream.Write (buffer, offset, len);
+			}
+
+			public void WriteSum ()
+			{
+				byte [] data = BitConverter.GetBytes (Value, false);
+				stream.Write (data, 0, data.Length);
+			}
+		}
+
 		public class Chunk {
 			public string Name;
 			protected byte [] data;
@@ -475,6 +666,12 @@ namespace FSpot.Png {
 				name_table ["IHDR"] = typeof (IhdrChunk);
 				name_table ["cHRM"] = typeof (ColorChunk);
 				name_table ["pHYs"] = typeof (PhysChunk);
+				name_table ["gAMA"] = typeof (GammaChunk);
+				name_table ["sRGB"] = typeof (StandardRgbChunk);
+			}
+			
+			protected Chunk ()
+			{
 			}
 			
 			public Chunk (string name, byte [] data) 
@@ -483,7 +680,6 @@ namespace FSpot.Png {
 				this.data = data;
 				Load (data);
 			}
-
 			
 			protected string GetString  (ref int i, System.Text.Encoding enc) 
 			{
@@ -504,6 +700,17 @@ namespace FSpot.Png {
 			{
 				
 			}
+			
+			public virtual void Save (System.IO.Stream stream)
+			{
+				byte [] name_bytes = System.Text.Encoding.ASCII.GetBytes (Name);
+				byte [] length_bytes = BitConverter.GetBytes ((uint)data.Length, false);
+				stream.Write (length_bytes, 0, length_bytes.Length);
+				Crc crc = new Crc (stream);
+				crc.Write (name_bytes);
+				crc.Write (data);
+				crc.WriteSum ();
+			}
 
 			public bool Critical {
 				get {
@@ -529,6 +736,16 @@ namespace FSpot.Png {
 				}
 			}
 
+			public bool CheckCrc (uint value)
+			{
+				byte [] name = System.Text.Encoding.ASCII.GetBytes (Name);
+				Crc crc = new Crc ();
+				crc.Write (name);
+				crc.Write (data);
+
+				return crc.Value == value;
+			}
+
 			public static Chunk Generate (string name, byte [] data)
 			{
 				System.Type t = (System.Type) name_table [name];
@@ -551,17 +768,27 @@ namespace FSpot.Png {
 				
 				byte [] buf = new byte [1024];
 				int inflate_length;
-				while ((inflate_length = inflater.Inflate (buf)) > 0) {
+				while ((inflate_length = inflater.Inflate (buf)) > 0)
 					output.Write (buf, 0, inflate_length);
-				}
 				
-				byte [] result = new byte [output.Length];
-				output.Position = 0;
-				output.Read (result, 0, result.Length);
 				output.Close ();
-				return result;
+				return output.ToArray ();
 			}
 
+			public static byte [] Deflate (byte [] input, int offset, int length)
+			{
+				System.IO.MemoryStream output = new System.IO.MemoryStream ();
+				Deflater deflater = new Deflater ();
+				deflater.SetInput (input, offset, length);
+				
+				byte [] buf = new byte [1024];
+				int deflate_length;
+				while ((deflate_length = deflater.Deflate (buf)) > 0)
+					output.Write (buf, 0, deflate_length);
+
+				output.Close ();
+				return output.ToArray ();
+			}
 		}
 
 		public class ChunkInflater {
@@ -589,7 +816,6 @@ namespace FSpot.Png {
 				int result = 0;
 				do {
 					Fill ();
-					int attempt = length - result;
 					result += inflater.Inflate (data, start + result, length - result);
 					//System.Console.WriteLine ("Attempting Second after fill Inflate {0} {1} {2}", attempt, result, length - result);
 				} while (result < length && chunks.Count > 0);
@@ -849,7 +1075,7 @@ namespace FSpot.Png {
 			}
 
 			IhdrChunk ihdr = (IhdrChunk) Chunks [0];
-			System.Console.WriteLine ("Attempting to to inflate image {0}.{1}({2}, {3})", ihdr.Color, ihdr.Depth, ihdr.Width, ihdr.Height);
+			System.Console.WriteLine ("Attempting to to inflate photo {0}.{1}({2}, {3})", ihdr.Color, ihdr.Depth, ihdr.Width, ihdr.Height);
 			ScanlineDecoder decoder = new ScanlineDecoder (ci, ihdr.GetScanlineLength (0), ihdr.Height);
 			decoder.Fill ();
 			//Gdk.Pixbuf pixbuf = decoder.GetPixbuf ();
@@ -920,27 +1146,17 @@ namespace FSpot.Png {
 			return pixbuf;
 		}
 
-		/*
-		public override Gdk.Pixbuf Load ()
-		{
-			return this.GetPixbuf ();
-		}
-		*/
+		private static byte [] magic = new byte [] { 137, 80, 78, 71, 13, 10, 26, 10 };
 
 	        void Load (System.IO.Stream stream)
 		{
 			byte [] heading = new byte [8];
+			byte [] crc_data = new byte [4];
 			stream.Read (heading, 0, heading.Length);
 
-			if (heading [0] != 137 ||
-			    heading [1] != 80 ||
-			    heading [2] != 78 ||
-			    heading [3] != 71 ||
-			    heading [4] != 13 ||
-			    heading [5] != 10 ||
-			    heading [6] != 26 ||
-			    heading [7] != 10)
-			    throw new System.Exception ("Invalid PNG magic number");
+			for (int i = 0; i < heading.Length; i++)
+				if (heading [i] != magic [i])
+					throw new System.Exception ("Invalid PNG magic number");
 
 			chunk_list = new System.Collections.ArrayList ();
 
@@ -951,34 +1167,55 @@ namespace FSpot.Png {
 				if (length > 0)
 					stream.Read (data, 0, data.Length);
 
-				stream.Read (heading, 0, 4);
-				uint crc = BitConverter.ToUInt32 (heading, 0, false);
+				stream.Read (crc_data, 0, 4);
+				uint crc = BitConverter.ToUInt32 (crc_data, 0, false);
 
 				Chunk chunk = Chunk.Generate (name, data);
-				//if (crc != chunk.Crc ())
-				//	throw new System.Exception ("chunk crc check failed");
+				if (! chunk.CheckCrc (crc))
+					throw new System.Exception ("chunk crc check failed");
 				
 				//System.Console.Write ("read one {0} {1}", chunk, chunk.Name);
 				chunk_list.Add (chunk);
 
-#if TEST_METADATA				
+#if false			       
 				if (chunk is TextChunk) {
 					TextChunk text = (TextChunk) chunk;
 					System.Console.Write (" Text Chunk {0} {1}", 
-							      text.Keyword, "", text.Text);
+							      text.Keyword, "", "");
 				}
 
 				TimeChunk time = chunk as TimeChunk;
 				if (time != null)
 					System.Console.Write(" Time {0}", time.Time);
 #endif
-				//System.Console.WriteLine ("");
+				System.Console.WriteLine ("");
 				
 				if (chunk.Name == "IEND")
 					break;
 			}
 		}
 
+
+		public void Save (System.IO.Stream stream)
+		{
+			stream.Write (magic, 0, magic.Length);
+			foreach (Chunk chunk in Chunks) {
+				chunk.Save (stream);
+			}
+		}
+
+		public void Save (string path)
+		{
+			string  temp_path = path + ".tmp.png";
+			using (System.IO.Stream output = System.IO.File.OpenWrite (temp_path)) {
+				Save (output);
+			}
+			if (FSpot.Unix.Rename (temp_path, path) < 0) {
+				System.IO.File.Delete (temp_path);
+				throw new System.Exception (System.String.Format ("Unable to rename {0} to {1}", temp_path, path));
+			}
+		}
+
 		public string LookupText (string keyword)
 		{
 			TextChunk chunk = LookupTextChunk (keyword);
@@ -998,6 +1235,63 @@ namespace FSpot.Png {
 			return null;	
 		}
 
+		public override Cms.Profile GetProfile ()
+		{
+			ColorChunk color = null;
+			IccpChunk icc = null;
+			GammaChunk gamma = null;
+			StandardRgbChunk srgb = null;
+			double gamma_value = 2.2;
+			ColorCIExyY red = new ColorCIExyY (0.64, 0.33, 1.0);
+			ColorCIExyY green = new ColorCIExyY (0.3, 0.6, 1.0);
+			ColorCIExyY blue = new ColorCIExyY (0.15, 0.06, 1.0);
+			ColorCIExyY whitepoint = new ColorCIExyY (0.3127, 0.329, 1.0);
+			ColorCIExyYTriple chroma = new ColorCIExyYTriple (red, green, blue);
+
+			System.Console.WriteLine ("Trying to get profile");
+
+			foreach (Chunk chunk in Chunks) {
+				if (color == null) 
+					color = chunk as ColorChunk;
+				if (icc == null)
+					icc = chunk as IccpChunk;
+				if (srgb == null)
+					srgb = chunk as StandardRgbChunk;
+				if (gamma == null)
+					gamma = chunk as GammaChunk;
+			}
+			
+			System.Console.WriteLine ("color: {0} icc: {1} srgb: {2} gamma: {3}", color, icc, srgb, gamma);
+
+			if (icc != null) {
+				try {
+					return new Profile (icc.Profile);
+				} catch (System.Exception ex) {
+					System.Console.WriteLine ("Error trying to decode embedded profile" + ex.ToString ());
+				}
+			}
+
+			if (srgb != null)
+				return Profile.CreateStandardRgb ();
+
+			if (gamma != null)
+				gamma_value = 1 / gamma.Gamma;
+			
+			if (color != null) {
+				whitepoint = new ColorCIExyY (color.WhiteX.Value, color.WhiteY.Value, 1.0);
+				red = new ColorCIExyY (color.RedX.Value, color.RedY.Value, 1.0);
+				green = new ColorCIExyY (color.GreenX.Value, color.GreenY.Value, 1.0);
+				blue = new ColorCIExyY (color.BlueX.Value, color.BlueY.Value, 1.0);
+				chroma = new ColorCIExyYTriple (red, green, blue);
+			}
+
+			if (color != null || gamma != null) {
+				GammaTable table = new GammaTable (1024, gamma_value);
+				return new Profile (whitepoint, chroma, new GammaTable [] {table, table, table});
+			}
+			
+			return null;
+		}
 
 		public override string Description {
 			get {
@@ -1010,6 +1304,62 @@ namespace FSpot.Png {
 			}
 		}
 
+		public void SetDescription (string description) 
+		{
+			TextChunk text = null;
+			text = LookupTextChunk ("Description");
+			
+			if (text != null)
+				text.SetText (description);
+			else 
+				Insert (new TextChunk ("Description", description));
+		}
+
+		public XmpFile GetXmp ()
+		{
+			TextChunk xmpchunk  = LookupTextChunk ("XML:com.adobe.xmp");
+			if (xmpchunk == null)
+				xmpchunk = LookupTextChunk ("XMP");
+
+			if (xmpchunk == null)
+				return null;
+			
+			using (MemoryStream stream = new MemoryStream (xmpchunk.TextData)) {
+				return new XmpFile (stream);
+			}
+		}
+
+		public void SetXmp (XmpFile xmp)
+		{
+			TextChunk text = null;
+
+			text = LookupTextChunk ("XML:com.adobe.xmp");
+			if (text != null)
+				Chunks.Remove (text);
+			
+			text = LookupTextChunk ("XMP");
+			if (text != null)
+				Chunks.Remove (text);
+
+			ItxtChunk itext = new ItxtChunk ("XML:com.adobe.xmp", "en", false);
+			MemoryStream stream = new MemoryStream ();
+			xmp.Save (stream);
+			itext.SetText (stream.ToArray ());
+			Insert (itext);
+		}
+
+		private void Insert (Chunk chunk)
+		{
+			// FIXME The point of this function is to enforce ordering constraints
+			// it obviously isn't complete right now.
+			if (chunk is IhdrChunk)
+				Chunks.Insert (0, chunk);
+			else if (chunk is TextChunk)
+				Chunks.Insert (1, chunk);
+			else
+				throw new System.Exception ("Uknown ordering for chunk");
+		}
+
 		public override System.DateTime Date {
 			get {
 				// FIXME: we should first try parsing the
@@ -1033,7 +1383,6 @@ namespace FSpot.Png {
 			}
 		}
 
-
 		public static void Main (string [] args) 
 		{
 			System.Collections.ArrayList failed = new System.Collections.ArrayList ();
